# configure environment to install controller from .deb and dev repo
class space_log(
  Optional[Tuple]   $extra_facts = {},
) {

  space_facts::addfacter { $name:
    extra_facts => $extra_facts,
  }

  contain '::space_log::rsyslog_severity'
  
  # for gem 
  ensure_resource('package', 'ruby', {'ensure' => 'present'})
  ensure_resource('package', 'ruby-dev', {'ensure' => 'present'})
  ensure_resource('package', 'gcc', {'ensure' => 'present'})
  ensure_resource('package', 'make', {'ensure' => 'present'})

  $controller_ip = hiera('controller_ip', '127.0.0.1')
  
  $logger=[ { 'host' => $controller_ip, 'port' => '24224'} ]
  
  $severity_levels = ['emergency', 'alert', 'critical', 'error', 'warning', 'notice', 'info', 'debug']

  include fluentd

  fluentd::plugin { 'faraday':
     type   => 'file',
     source => "puppet:///modules/${module_name}/faraday-0.15.4.gem",
   }

   exec { 'grafana_loki_install':
    command => "td-agent-gem install -N --conservative --minimal-deps --no-document --force --local /etc/puppetveil/code/environments/production/modules/space_log/files/fluent-plugin-grafana-loki-1.2.20.gem",
    unless  => "td-agent-gem list --local | grep fluent-plugin-grafana-loki | grep 1.2.20",
  }

  fluentd::plugin { 'fluent-plugin-grafana-loki':
     type   => 'file',
     source => "puppet:///modules/${module_name}/fluent-plugin-grafana-loki-1.2.20.gem",
   }

  if defined('$is_controller_media') {
    if $is_controller_media == '0' {
      $tag = 'remote'
    } else {
      $tag = 'local'
    }
  } else {
    $tag = 'remote'
  }

  fluentd::source { 'collector':
    priority => 10,
    config   => {
      'bind' => '::',
      'type' => 'forward',
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'rewrite_local_tag':
    priority => 50,
    pattern => 'local.**',
    config   => {
      '@type' => 'rewrite_tag_filter',
      'remove_tag_prefix' => 'local',
      rule => {
        key => 'host',
        pattern => '.+',
        tag => '${tag}'
      }
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'rewrite_remote_tag':
    priority => 50,
    pattern => 'remote.**',
    config   => {
      '@type' => 'rewrite_tag_filter',
      'remove_tag_prefix' => 'remote',
      rule => {
        key => 'host',
        pattern => '.+',
        tag => '${tag}'
      }
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'system.syslog_loki':
    priority => 55,
    pattern => 'system.syslog',
    config  => {
      '@type' => 'loki',
      'url' => 'http://localhost:3100',
      'extra_labels' => '{"name":"system.syslog"}',
      'buffer' => {"flush_interval" => "10s", "flush_at_shutdown" => "true"},
      'buffer_chunk_limit' => '1m',
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'forward_system':
    priority => 90,
    pattern => '**',
    config   => {
      '@type'    => 'forward',
      'server' => $logger,
    },
    notify => Class['fluentd::service']
  }

  $severity_levels.each |String $level| {
    # собираем логи по уровням важности
    fluentd::source { "syslog_${level}":
      config => {
        '@type' => 'tail',
        format => 'syslog',
        tag => "${level}.${tag}.system.syslog",
        encoding => 'UTF-8',
        path => "/var/log/${level}.system.syslog",
        pos_file => "/var/log/td-agent/td-agent.${level}.syslog.pos",
      },
      notify => Class['fluentd::service']
    }

    fluentd::source { "syslog_${level}_1":
      config => {
        '@type' => 'tail',
        format => 'syslog',
        tag => "${level}.${tag}.system.syslog",
        encoding => 'UTF-8',
        path => "/var/log/${level}.system.syslog.1",
        pos_file => "/var/log/td-agent/td-agent.${level}.syslog_1.pos",
      },
      notify => Class['fluentd::service']
    }

    # добавляем поле указывающее уровень важности
    fluentd::filter { "transform_${level}_syslog":
      priority => 20,
      pattern => "${level}.**",
      config => {
        '@type' => 'record_transformer',
        record => {
          levelname => upcase($level)
        }
      },
      notify => Class['fluentd::service']
    }

    # чистим теги
    fluentd::match { "rewrite_${level}_tag":
      priority => 25,
      pattern => "${level}.**",
      config   => {
        '@type' => 'rewrite_tag_filter',
        'remove_tag_prefix' => $level,
        rule => {
          key => 'host',
          pattern => '.+',
          tag => '${tag}'
        }
      },
      notify => Class['fluentd::service']
    }
  }

  fluentd::source { 'syslog_dpkg':
    config => {
      '@type' => 'tail',
      format => '/(?<time>^[^ ]+ [^ ]+) (?<message>.*)/',
      time_format => '%Y-%m-%d %H:%M:%S',
      tag => sprintf('%s.dpkg', $tag),
      path => '/var/log/dpkg.log',
      pos_file => '/var/log/td-agent/td-agent.dpkg.pos',
    },
    notify => Class['fluentd::service']
  }

  fluentd::source { 'veil_cli_log':
    config => {
      '@type' => 'tail',
      format => 'json',
      tag => sprintf('%s.veil.cli', $tag),
      path => '/var/log/veil/cli/cli.log',
      pos_file => '/var/log/td-agent/veil_cli.log.pos',
    },
    notify => Class['fluentd::service']
  }

  # добавляем поле ident для dpkg логов
  fluentd::filter { 'transform_dpkg':
    priority => 20,
    pattern => sprintf('%s.dpkg', $tag),
    config => {
      '@type' => 'record_transformer',
      record => {
        ident => 'dpkg'
      }
    },
    notify => Class['fluentd::service']
  }

  # Добавляем настройку фильтра добавляющего тег и host/hostname для логов
  fluentd::filter { 'transform_add_host':
    priority => 30,
    pattern => sprintf('%s.**', $tag),
    config => {
      '@type' => 'record_transformer',
      enable_ruby => '',
      record => {
        hostname => '"#{Socket.gethostname}"',
        host => '"#{Socket.gethostname; Socket.getifaddrs.map(&:addr).select(&:ipv4?).map(&:ip_address) - [\'127.0.0.1\']}"'
      }
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'rewrite_dpkg_tag':
    priority => 40,
    pattern => sprintf('%s.dpkg', $tag),
    config => {
      '@type' => 'rewrite_tag_filter',
      rule => {
        key => 'message',
        pattern => '.+',
        tag => sprintf('%s.system.syslog', $tag)
      }
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'veil.cli_loki':
    priority => 55,
    pattern => 'veil.cli',
    config  => {
      '@type' => 'loki',
      'url' => 'http://localhost:3100',
      'extra_labels' => '{"name":"veil.cli"}',
      'buffer' => {"flush_interval" => "10s", "flush_at_shutdown" => "true"},
      'buffer_chunk_limit' => '1m',
      'line_format' => 'json'
    },
    notify => Class['fluentd::service']
  }

  fluentd::match { 'fluent_loki':
    priority => 55,
    pattern => 'fluent.*',
    config  => {
      '@type' => 'loki',
      'url' => 'http://localhost:3100',
      'label' => {"first" => "@FLUENT_LOG"},
      'extra_labels' => '{"name":"fluentd"}',
      'buffer' => {"flush_interval" => "10s", "flush_at_shutdown" => "true"},
      'buffer_chunk_limit' => '1m',
    },
    notify => Class['fluentd::service']
  }

  if defined('$is_controller_media') {
    # Controller specific section
    if $is_controller_media == '1' {
      fluentd::source { 'space_controller_log':
        config => {
          '@type' => 'tail',
          format => 'json',
          tag => 'local.veil.controller',
          path => '/var/log/veil/controller/controller.log',
          pos_file => '/var/log/td-agent/space_controller.log.pos',
        },
        notify => Class['fluentd::service']
      }

      fluentd::source { 'veil_django_log':
        config => {
          '@type' => 'tail',
          format => 'json',
          tag => 'local.veil.controller',
          path => '/var/log/veil/controller/django.log',
          pos_file => '/var/log/td-agent/veil_django.log.pos',
        },
        notify => Class['fluentd::service']
      }

      fluentd::source { 'veil_async_log':
        config => {
          '@type' => 'tail',
          format => 'json',
          tag => 'local.veil.controller',
          path => '/var/log/veil/controller/veil_async.log',
          pos_file => '/var/log/td-agent/veil_async.log.pos',
        },
        notify => Class['fluentd::service']
      }

      fluentd::match { 'veil.controller_loki':
        priority => 55,
        pattern => 'veil.controller',
        config  => {
          '@type' => 'loki',
          'url' => 'http://localhost:3100',
          'extra_labels' => '{"name":"veil.controller"}',
          'buffer' => {"flush_interval" => "10s", "flush_at_shutdown" => "true"},
          'buffer_chunk_limit' => '1m',
          'line_format' => 'json'
        },
        notify => Class['fluentd::service']
      }

      # TEST3
      file { "/var/lib/ecp-veil/tests/ctr_init_test3.sh":
        ensure  => present,
        content => template("${module_name}/ctr_init_test3.sh.erb"),
        mode => '0550',
      }
    }
  }
  if defined('$is_node_media') {
  # Node specific section
    if $is_node_media == '1' {
      fluentd::match { 'veil.node_loki':
        priority => 55,
        pattern => 'veil.node',
        config  => {
          '@type' => 'loki',
          'url' => 'http://localhost:3100',
          'extra_labels' => '{"name":"veil.node"}',
          'buffer' => {"flush_interval" => "10s", "flush_at_shutdown" => "true"},
          'buffer_chunk_limit' => '1m',
          'line_format' => 'json'
        },
        notify => Class['fluentd::service']
      }

      fluentd::source { 'veil_node_log':
        config => {
          '@type' => 'tail',
          format => 'json',
          tag => sprintf('%s.veil.node', $tag),
          path => '/var/log/veil/node/node.log',
          pos_file => '/var/log/td-agent/veil_node.log.pos',
        },
        notify => Class['fluentd::service']
      }
      
      file { "/var/lib/ecp-veil/tests/node_init_test3.sh":
        ensure  => present,
        content => template("${module_name}/node_init_test3.sh.erb"),
        mode => '0550',
      }
    }
  }
}
